@@ -12,7 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from tempest_lib import decorators
 from tempest_lib import exceptions
 
 from mistralclient.tests.functional.cli import base
@@ -43,7 +42,7 @@ class SimpleMistralCLITests(base.MistralCLIAuth):
             self.mistral('workflow-list'))
         self.assertTableStruct(
             workflows,
-            ['Name', 'Tags', 'Input', 'Created at', 'Updated at']
+            ['ID', 'Name', 'Tags', 'Input', 'Created at', 'Updated at']
         )
 
     def test_executions_list(self):
@@ -51,7 +50,8 @@ class SimpleMistralCLITests(base.MistralCLIAuth):
             self.mistral('execution-list'))
         self.assertTableStruct(
             executions,
-            ['ID', 'Workflow', 'State', 'Created at', 'Updated at']
+            ['ID', 'Workflow name', 'Workflow ID', 'State', 'Created at',
+             'Updated at']
         )
 
     def test_tasks_list(self):
@@ -183,6 +183,27 @@ class WorkbookCLITests(base_v2.MistralClientTestBase):
             'workbook-get-definition', params=wb_name)
         self.assertNotIn('404 Not Found', definition)
 
+    def test_workbook_validate_with_valid_def(self):
+        wb = self.mistral_admin(
+            'workbook-validate', params=self.wb_def)
+        wb_valid = self.get_value_of_field(wb, 'Valid')
+        wb_error = self.get_value_of_field(wb, 'Error')
+
+        self.assertEqual('True', wb_valid)
+        self.assertEqual('None', wb_error)
+
+    def test_workbook_validate_with_invalid_def(self):
+        self.create_file('wb.yaml',
+                         'name: wb\n')
+        wb = self.mistral_admin(
+            'workbook-validate', params='wb.yaml')
+
+        wb_valid = self.get_value_of_field(wb, 'Valid')
+        wb_error = self.get_value_of_field(wb, 'Error')
+
+        self.assertEqual('False', wb_valid)
+        self.assertNotEqual('None', wb_error)
+
 
 class WorkflowCLITests(base_v2.MistralClientTestBase):
     """Test suite checks commands to work with workflows."""
@@ -221,37 +242,105 @@ class WorkflowCLITests(base_v2.MistralClientTestBase):
     def test_workflow_update(self):
         wf = self.workflow_create(self.wf_def)
         wf_name = wf[0]['Name']
+        wf_id = wf[0]['ID']
 
         created_wf_info = self.get_item_info(
-            get_from=wf, get_by='Name', value=wf_name)
+            get_from=wf,
+            get_by='Name',
+            value=wf_name
+        )
 
+        # Update a workflow with definition unchanged.
         upd_wf = self.mistral_admin(
-            'workflow-update', params='{0}'.format(self.wf_def))
+            'workflow-update',
+            params='{0}'.format(self.wf_def)
+        )
+
         self.assertTableStruct(upd_wf, ['Name', 'Created at', 'Updated at'])
 
         updated_wf_info = self.get_item_info(
-            get_from=upd_wf, get_by='Name', value=wf_name)
+            get_from=upd_wf,
+            get_by='Name',
+            value=wf_name
+        )
 
         self.assertEqual(wf_name, upd_wf[0]['Name'])
+        self.assertEqual(
+            created_wf_info['Created at'].split(".")[0],
+            updated_wf_info['Created at']
+        )
+        self.assertEqual(
+            created_wf_info['Updated at'],
+            updated_wf_info['Updated at']
+        )
 
-        self.assertEqual(created_wf_info['Created at'].split(".")[0],
-                         updated_wf_info['Created at'])
-        self.assertEqual(created_wf_info['Updated at'],
-                         updated_wf_info['Updated at'])
-
+        # Update a workflow with definition changed.
         upd_wf = self.mistral_admin(
-            'workflow-update', params='{0}'.format(self.wf_with_delay_def))
+            'workflow-update',
+            params='{0}'.format(self.wf_with_delay_def)
+        )
+
         self.assertTableStruct(upd_wf, ['Name', 'Created at', 'Updated at'])
 
         updated_wf_info = self.get_item_info(
-            get_from=upd_wf, get_by='Name', value=wf_name)
+            get_from=upd_wf,
+            get_by='Name',
+            value=wf_name
+        )
 
         self.assertEqual(wf_name, upd_wf[0]['Name'])
+        self.assertEqual(
+            created_wf_info['Created at'].split(".")[0],
+            updated_wf_info['Created at']
+        )
+        self.assertNotEqual(
+            created_wf_info['Updated at'],
+            updated_wf_info['Updated at']
+        )
 
-        self.assertEqual(created_wf_info['Created at'].split(".")[0],
-                         updated_wf_info['Created at'])
-        self.assertNotEqual(created_wf_info['Updated at'],
-                            updated_wf_info['Updated at'])
+        # Update a workflow with uuid.
+        upd_wf = self.mistral_admin(
+            'workflow-update',
+            params='{0} --id {1}'.format(self.wf_with_delay_def, wf_id)
+        )
+
+        self.assertTableStruct(upd_wf, ['Name', 'Created at', 'Updated at'])
+
+        updated_wf_info = self.get_item_info(
+            get_from=upd_wf,
+            get_by='ID',
+            value=wf_id
+        )
+
+        self.assertEqual(wf_name, upd_wf[0]['Name'])
+        self.assertEqual(
+            created_wf_info['Created at'].split(".")[0],
+            updated_wf_info['Created at']
+        )
+        self.assertNotEqual(
+            created_wf_info['Updated at'],
+            updated_wf_info['Updated at']
+        )
+
+    def test_workflow_update_truncate_input(self):
+        input_value = "very_long_input_parameter_name_that_should_be_truncated"
+        wf_def = """
+        version: "2.0"
+        workflow1:
+          input:
+            - {0}
+          tasks:
+            task1:
+              action: std.noop
+        """.format(input_value)
+        self.create_file('wf.yaml', wf_def)
+        self.workflow_create('wf.yaml')
+        upd_wf = self.mistral_admin(
+            'workflow-update', params='wf.yaml')
+        upd_wf_info = self.get_item_info(
+            get_from=upd_wf, get_by='Name', value='workflow1')
+
+        self.assertEqual(upd_wf_info['Input'][:-3], input_value[:25])
 
     def test_workflow_get(self):
         created = self.workflow_create(self.wf_def)
@@ -261,6 +350,15 @@ class WorkflowCLITests(base_v2.MistralClientTestBase):
         fetched_wf_name = self.get_value_of_field(fetched, 'Name')
         self.assertEqual(wf_name, fetched_wf_name)
 
+    def test_workflow_get_with_id(self):
+        created = self.workflow_create(self.wf_def)
+        wf_name = created[0]['Name']
+        wf_id = created[0]['ID']
+
+        fetched = self.mistral_admin('workflow-get', params=wf_id)
+        fetched_wf_name = self.get_value_of_field(fetched, 'Name')
+        self.assertEqual(wf_name, fetched_wf_name)
+
     def test_workflow_get_definition(self):
         wf = self.workflow_create(self.wf_def)
         wf_name = wf[0]['Name']
@@ -269,6 +367,27 @@ class WorkflowCLITests(base_v2.MistralClientTestBase):
             'workflow-get-definition', params=wf_name)
         self.assertNotIn('404 Not Found', definition)
 
+    def test_workflow_validate_with_valid_def(self):
+        wf = self.mistral_admin(
+            'workflow-validate', params=self.wf_def)
+        wf_valid = self.get_value_of_field(wf, 'Valid')
+        wf_error = self.get_value_of_field(wf, 'Error')
+
+        self.assertEqual('True', wf_valid)
+        self.assertEqual('None', wf_error)
+
+    def test_workflow_validate_with_invalid_def(self):
+        self.create_file('wf.yaml',
+                         'name: wf\n')
+        wf = self.mistral_admin(
+            'workflow-validate', params='wf.yaml')
+
+        wf_valid = self.get_value_of_field(wf, 'Valid')
+        wf_error = self.get_value_of_field(wf, 'Error')
+
+        self.assertEqual('False', wf_valid)
+        self.assertNotEqual('None', wf_error)
+
 
 class ExecutionCLITests(base_v2.MistralClientTestBase):
     """Test suite checks commands to work with executions."""
@@ -295,17 +414,19 @@ class ExecutionCLITests(base_v2.MistralClientTestBase):
         exec_id = self.get_value_of_field(execution, 'ID')
         self.assertTableStruct(execution, ['Field', 'Value'])
 
-        wf = self.get_value_of_field(execution, 'Workflow')
+        wf_name = self.get_value_of_field(execution, 'Workflow name')
+        wf_id = self.get_value_of_field(execution, 'Workflow ID')
         created_at = self.get_value_of_field(execution, 'Created at')
         description = self.get_value_of_field(execution, 'Description')
 
-        self.assertEqual(self.direct_wf['Name'], wf)
+        self.assertEqual(self.direct_wf['Name'], wf_name)
+        self.assertIsNotNone(wf_id)
         self.assertIsNotNone(created_at)
-        self.assertEqual(description, "execution test")
+        self.assertEqual("execution test", description)
 
         execs = self.mistral_admin('execution-list')
         self.assertIn(exec_id, [ex['ID'] for ex in execs])
-        self.assertIn(wf, [ex['Workflow'] for ex in execs])
+        self.assertIn(wf_name, [ex['Workflow name'] for ex in execs])
 
         self.mistral_admin('execution-delete', params=exec_id)
 
@@ -343,7 +464,7 @@ class ExecutionCLITests(base_v2.MistralClientTestBase):
 
         description = self.get_value_of_field(execution, 'Description')
 
-        self.assertEqual(description, "execution update test")
+        self.assertEqual("execution update test", description)
 
     def test_execution_get(self):
         execution = self.execution_create(self.direct_wf['Name'])
@@ -353,10 +474,12 @@ class ExecutionCLITests(base_v2.MistralClientTestBase):
             'execution-get', params='{0}'.format(exec_id))
 
         gotten_id = self.get_value_of_field(execution, 'ID')
-        wf = self.get_value_of_field(execution, 'Workflow')
+        wf_name = self.get_value_of_field(execution, 'Workflow name')
+        wf_id = self.get_value_of_field(execution, 'Workflow ID')
 
+        self.assertIsNotNone(wf_id)
         self.assertEqual(exec_id, gotten_id)
-        self.assertEqual(self.direct_wf['Name'], wf)
+        self.assertEqual(self.direct_wf['Name'], wf_name)
 
     def test_execution_get_input(self):
         execution = self.execution_create(self.direct_wf['Name'])
@@ -374,6 +497,74 @@ class ExecutionCLITests(base_v2.MistralClientTestBase):
 
         self.assertEqual([], ex_output)
 
+    def test_executions_list_with_pagination(self):
+
+        execution1 = self.mistral_admin(
+            'execution-create',
+            params='{0} -d "a"'.format(self.direct_wf['Name'])
+        )
+
+        execution2 = self.mistral_admin(
+            'execution-create',
+            params='{0} -d "b"'.format(self.direct_wf['Name'])
+        )
+
+        executions = self.mistral_cli(
+            True,
+            'execution-list',
+            params="--limit 1"
+        )
+
+        self.assertEqual(1, len(executions))
+
+        exec_id_1 = self.get_value_of_field(execution1, 'ID')
+        exec_id_2 = self.get_value_of_field(execution2, 'ID')
+        executions = self.mistral_cli(
+            True,
+            'execution-list',
+            params="--marker %s" % exec_id_1
+        )
+        self.assertNotIn(exec_id_1, [ex['ID'] for ex in executions])
+        self.assertIn(exec_id_2, [ex['ID'] for ex in executions])
+
+        executions = self.mistral_cli(
+            True,
+            'execution-list',
+            params="--sort_keys Description"
+        )
+
+        self.assertIn(exec_id_1, [ex['ID'] for ex in executions])
+        self.assertIn(exec_id_2, [ex['ID'] for ex in executions])
+
+        ex1_index = -1
+        ex2_index = -1
+        for idx, ex in enumerate(executions):
+            if ex['ID'] == exec_id_1:
+                ex1_index = idx
+            elif ex['ID'] == exec_id_2:
+                ex2_index = idx
+
+        self.assertTrue(ex1_index < ex2_index)
+
+        executions = self.mistral_cli(
+            True,
+            'execution-list',
+            params="--sort_keys Description --sort_dirs=desc"
+        )
+
+        self.assertIn(exec_id_1, [ex['ID'] for ex in executions])
+        self.assertIn(exec_id_2, [ex['ID'] for ex in executions])
+
+        ex1_index = -1
+        ex2_index = -1
+        for idx, ex in enumerate(executions):
+            if ex['ID'] == exec_id_1:
+                ex1_index = idx
+            elif ex['ID'] == exec_id_2:
+                ex2_index = idx
+
+        self.assertTrue(ex1_index > ex2_index)
+
 
 class CronTriggerCLITests(base_v2.MistralClientTestBase):
     """Test suite checks commands to work with cron-triggers."""
@@ -498,14 +689,14 @@ class ActionCLITests(base_v2.MistralClientTestBase):
         action_2 = self.get_item_info(
             get_from=init_acts, get_by='Name', value='farewell')
 
-        self.assertEqual(action_1['Tags'], '<none>')
-        self.assertEqual(action_2['Tags'], '<none>')
+        self.assertEqual('<none>', action_1['Tags'])
+        self.assertEqual('<none>', action_2['Tags'])
 
-        self.assertEqual(action_1['Is system'], 'False')
-        self.assertEqual(action_2['Is system'], 'False')
+        self.assertEqual('False', action_1['Is system'])
+        self.assertEqual('False', action_2['Is system'])
 
-        self.assertEqual(action_1['Input'], 'name')
-        self.assertEqual(action_2['Input'], 'None')
+        self.assertEqual('name', action_1['Input'])
+        self.assertEqual('None', action_2['Input'])
 
         acts = self.mistral_admin('action-list')
         self.assertIn(action_1['Name'], [action['Name'] for action in acts])
@@ -542,13 +733,31 @@ class ActionCLITests(base_v2.MistralClientTestBase):
         updated_action = self.get_item_info(
             get_from=acts, get_by='Name', value='greeting')
 
-        self.assertEqual(updated_action['Tags'], 'tag, tag1')
+        self.assertEqual('tag, tag1', updated_action['Tags'])
         self.assertEqual(created_action['Created at'].split(".")[0],
                          updated_action['Created at'])
         self.assertEqual(created_action['Name'], updated_action['Name'])
         self.assertNotEqual(created_action['Updated at'],
                             updated_action['Updated at'])
 
+    def test_action_update_truncate_input(self):
+        input_value = "very_long_input_parameter_name_that_should_be_truncated"
+        act_def = """
+        version: "2.0"
+        action1:
+          input:
+            - {0}
+          base: std.noop
+        """.format(input_value)
+        self.create_file('action.yaml', act_def)
+        self.action_create('action.yaml')
+        upd_act = self.mistral_admin(
+            'action-update', params='action.yaml')
+        upd_act_info = self.get_item_info(
+            get_from=upd_act, get_by='Name', value='action1')
+
+        self.assertEqual(upd_act_info['Input'][:-3], input_value[:25])
+
     def test_action_get_definition(self):
         self.action_create(self.act_def)
 
@@ -585,6 +794,29 @@ class EnvironmentCLITests(base_v2.MistralClientTestBase):
         envs = self.mistral_admin('environment-list')
         self.assertNotIn(env_name, [en['Name'] for en in envs])
 
+    def test_environment_create_without_description(self):
+        self.create_file('env_without_des.yaml',
+                         'name: env\n'
+                         'variables:\n'
+                         '  var: "value"')
+
+        env = self.mistral_admin('environment-create',
+                                 params='env_without_des.yaml')
+
+        env_name = self.get_value_of_field(env, 'Name')
+        env_desc = self.get_value_of_field(env, 'Description')
+
+        self.assertTableStruct(env, ['Field', 'Value'])
+
+        envs = self.mistral_admin('environment-list')
+        self.assertIn(env_name, [en['Name'] for en in envs])
+        self.assertIn(env_desc, 'None')
+
+        self.mistral_admin('environment-delete', params='env')
+
+        envs = self.mistral_admin('environment-list')
+        self.assertNotIn(env_name, [en['Name'] for en in envs])
+
     def test_environment_update(self):
         env = self.environment_create('env.yaml')
         env_name = self.get_value_of_field(env, 'Name')
@@ -593,7 +825,7 @@ class EnvironmentCLITests(base_v2.MistralClientTestBase):
         env_updated_at = self.get_value_of_field(env, 'Updated at')
 
         self.assertIsNotNone(env_created_at)
-        self.assertEqual(env_updated_at, 'None')
+        self.assertEqual('None', env_updated_at)
 
         self.create_file('env_upd.yaml',
                          'name: env\n'
@@ -611,27 +843,10 @@ class EnvironmentCLITests(base_v2.MistralClientTestBase):
 
         self.assertEqual(env_name, updated_env_name)
         self.assertNotEqual(env_desc, updated_env_desc)
-        self.assertEqual(updated_env_desc, 'Updated env')
+        self.assertEqual('Updated env', updated_env_desc)
         self.assertEqual(env_created_at.split('.')[0], updated_env_created_at)
         self.assertIsNotNone(updated_env_updated_at)
 
-    @decorators.skip_because(bug="1418545")
-    def test_environment_update_work_as_create(self):
-        env = self.mistral_admin('environment-update', params='env.yaml')
-        env_name = self.get_value_of_field(env, 'Name')
-        env_desc = self.get_value_of_field(env, 'Description')
-
-        self.assertTableStruct(env, ['Field', 'Value'])
-
-        envs = self.mistral_admin('environment-list')
-        self.assertIn(env_name, [en['Name'] for en in envs])
-        self.assertIn(env_desc, [en['Description'] for en in envs])
-
-        self.mistral_admin('environment-delete', params=env_name)
-
-        envs = self.mistral_admin('environment-list')
-        self.assertNotIn(env_name, [en['Name'] for en in envs])
-
     def test_environment_get(self):
         env = self.environment_create('env.yaml')
         env_name = self.get_value_of_field(env, 'Name')
@@ -677,8 +892,8 @@ class ActionExecutionCLITests(base_v2.MistralClientTestBase):
             act_ex_from_list['ID'],
             self.get_value_of_field(act_ex, 'ID')
         )
-        self.assertEqual(wf_name, self.direct_wf['Name'])
-        self.assertEqual(status, 'SUCCESS')
+        self.assertEqual(self.direct_wf['Name'], wf_name)
+        self.assertEqual('SUCCESS', status)
 
     def test_act_execution_create_delete(self):
         action_ex = self.mistral_admin(
@@ -1071,6 +1286,12 @@ class NegativeCLITests(base_v2.MistralClientTestBase):
                           self.mistral_admin,
                           'environment-get')
 
+    def test_env_get_nonexistent(self):
+        self.assertRaises(exceptions.CommandFailed,
+                          self.mistral_admin,
+                          'environment-get',
+                          params='nonexist')
+
     def test_env_create_same_name(self):
         self.create_file('env.yaml',
                          'name: env\n'
@@ -1083,6 +1304,12 @@ class NegativeCLITests(base_v2.MistralClientTestBase):
                           self.environment_create,
                           'env.yaml')
 
+    def test_env_create_empty(self):
+        self.create_file('env.yaml')
+        self.assertRaises(exceptions.CommandFailed,
+                          self.environment_create,
+                          'env.yaml')
+
     def test_env_create_with_wrong_path_to_definition(self):
         self.assertRaises(exceptions.CommandFailed,
                           self.mistral_admin,
@@ -1101,6 +1328,22 @@ class NegativeCLITests(base_v2.MistralClientTestBase):
                           'environment-update',
                           params='env')
 
+    def test_env_update_empty(self):
+        self.create_file('env.yaml')
+        self.assertRaises(exceptions.CommandFailed,
+                          self.mistral_admin,
+                          'environment-update',
+                          params='env')
+
+    def test_env_update_nonexistant_env(self):
+        self.create_file('env.yaml',
+                         'name: env'
+                         'variables:\n  var: "value"')
+        self.assertRaises(exceptions.CommandFailed,
+                          self.mistral_admin,
+                          'environment-update',
+                          params='env.yaml')
+
     def test_env_create_without_name(self):
         self.create_file('env.yaml',
                          'variables:\n  var: "value"')
